/*
 * @(#) $Id:  $
 *
 * Copyright 2009/2010 by sIT Solutions,
 * A-1110 Wien, Geiselbergstr.21-25.
 * All rights reserved.
 *
 */
package org.openxma.dsl.platform.exceptions;

/**
 * An application exception that communicates that object creation/update was rejected
 * due to identity with an already persisted business object. Usually a NonUniqueException
 * is caused by unique constraint violation.<br/>
 * A bean name and a default message are always required when creating a NonUniqueException.
 * The conflicting properties should be specified but can be omitted if unknown.
 * 
 * <p>There are error codes for the basic occurrences of this exception but a more specific
 * error code can be defined, which will be used as resource bundle key to be resolved to a more
 * specific error message. See {@link MessageProvider} and its implementations for message
 * resolution.
 * 
 */
public class NonUniqueException extends ApplicationException {

    private static final long serialVersionUID = 1L;

    public static final String ERR_NON_UNIQUE = "error.non_unique";
    public static final String ERR_NON_UNIQUE_FIELDS = "error.non_unique.field";
    
    private final String beanName;
    private final String[] propertyNames;

    public NonUniqueException(String beanName, String defaultMessage) {
        this(ERR_NON_UNIQUE, beanName, null, defaultMessage);
    }

    public NonUniqueException(String beanName, String[] propertyNames, String defaultMessage) {
        this(ERR_NON_UNIQUE_FIELDS, beanName, propertyNames, defaultMessage);
    }

    public NonUniqueException(String errorCode, String beanName, String[] propertyNames,
            String defaultMessage) {
        super(errorCode, new Object[] { beanName, propertyNames }, defaultMessage);
        this.beanName = beanName;
        this.propertyNames = propertyNames;
    }

    String getBeanName() {
        return beanName;
    }

    String[] getPropertyNames() {
        return propertyNames;
    }
}
