/*
 * @(#) $Id:  $
 *
 * Copyright 2009/2010 by sIT Solutions,
 * A-1110 Wien, Geiselbergstr.21-25.
 * All rights reserved.
 *
 */
package org.openxma.dsl.platform.hibernate.type;

import java.io.IOException;
import java.io.Serializable;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.hibernate.HibernateException;
import org.hibernate.usertype.UserType;
import java.sql.Types;

import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;
import org.openxma.dsl.platform.util.ByteArrayContainer;

public class ByteArrayUserType implements UserType, Serializable {

    /**
     * 
     */
    private static final long serialVersionUID = -2496895380400888283L;

    @SuppressWarnings("rawtypes")
    private static final Class returnedClass = ByteArrayContainer.class;

    private static final int[] SQL_TYPES = { java.sql.Types.VARBINARY };

    public int[] sqlTypes() {
        return SQL_TYPES;
    }

    @SuppressWarnings("rawtypes")
    public Class returnedClass() {
        return returnedClass;
    }

    public boolean equals(Object x, Object y) throws HibernateException {
        if (x == null && y == null) {
            return true;
        } else if (x == null && y != null) {
            return false;
        } else {
            return x.equals(y);
        }

    }

    public int hashCode(Object x) throws HibernateException {
        return x.hashCode();

    }

    public Object nullSafeGet(ResultSet rs, String[] names, Object owner) throws HibernateException, SQLException {
        byte[] bytes = rs.getBytes(names[0]);
        if (bytes != null) {
            return new ByteArrayContainer(bytes);
        }
        return null;
    }

    public void nullSafeSet(PreparedStatement st, Object value, int index) throws HibernateException, SQLException {
        ByteArrayContainer bac = (ByteArrayContainer) value;
        byte[] b = null;
        if (bac != null) {
            b = bac.getData();
            st.setBytes(index, b);
        } else {
            st.setNull(index, Types.VARBINARY);
        }

    }

    public Object deepCopy(Object value) throws HibernateException {
        if (value == null) {
            return null;
        }

        ByteArrayContainer bac = (ByteArrayContainer) value;
        byte[] data = bac.getData();
        byte[] copy = new byte[data.length];
        System.arraycopy(data, 0, copy, 0, data.length);

        return new ByteArrayContainer(copy);
    }

    public boolean isMutable() {
        return false;
    }

    public Serializable disassemble(Object value) throws HibernateException {
        return new BASE64Encoder().encode((byte[]) value);
    }

    public Object assemble(Serializable cached, Object owner) throws HibernateException {
        try {
            return new BASE64Decoder().decodeBuffer((String) cached);
        } catch (IOException e) {
            throw new HibernateException("Could not assemble String to byte[] (Base64)", e);
        }
    }

    public Object replace(Object original, Object target, Object owner) throws HibernateException {
        return deepCopy(original);
    }

}
