package org.openxma.dsl.platform.xma.client.validator;

import java.util.Locale;
import java.util.StringTokenizer;

import at.spardat.enterprise.fmt.ADateFmt;
import at.spardat.enterprise.fmt.ATimeStampFmt;

public class DateTimeValidator extends ATimeStampFmt {
    
    private final static int DEFAULT_STYLE = ADateFmt.SHORT;
    
    /**
     * The DateIimeValidator is a special timestamp-validator which can be
     * constructed by specifying the styles of the date and the style of the
     * time in one single String separated by a comma.
     * 
     * Examples:
     *   DateTimeValidator("medium,medium",l);  // medium date and medium time
     *   DateTimeValidator("medium,short",l);   // medium date and short time
     *   DateTimeValidator("medium",l);         // medium date and short time (default for time is short)   
     *   DateTimeValidator("",l);               // short date and short time (default for date is short)        
     *   DateTimeValidator(null,l);             // short date and short time (null is allowed)     
     *   
     * Note: This timestamp-validator (as any other timestamp-validator) can also used for java.util.Date objects.  
     *   
     * @param format
     * @param l
     */
    public DateTimeValidator(String formats, Locale l) {        
        this(parseFormat(formats),l);
    }
    
    protected DateTimeValidator(int[] styles, Locale l) {
        super(styles[0],styles[1],l);        
    }
    
    private static int[] parseFormat(String format) {
        int styles[] = new int[2];
        if (format != null) {
            StringTokenizer tokenizer = new StringTokenizer(format,",");
            for (int i=0;i < styles.length;i++) {
                if (tokenizer.hasMoreElements()) {
                    styles[i] = createStyleFromFormat(tokenizer.nextToken());
                } else {
                    styles[i] = DEFAULT_STYLE;
                }
            }
        } else {
            styles[0] = DEFAULT_STYLE;
            styles[1] = DEFAULT_STYLE;            
        }
        return styles;
    }
    
    private static int createStyleFromFormat(String format) {
        if ("short".equalsIgnoreCase(format)) return ADateFmt.SHORT;
        if ("medium".equalsIgnoreCase(format)) return ADateFmt.MEDIUM;      
        if ("long".equalsIgnoreCase(format)) return ADateFmt.LONG;      
        if ("full".equalsIgnoreCase(format)) return ADateFmt.FULL;
        
        /* Default is SHORT */
        if (format == null || format.length()==0) return ADateFmt.SHORT;
        
        /* Not supported */
        throw new RuntimeException("Format not supported: "+format);
    }
}

    